use clap::Parser;
use core::panic;
use std::collections::HashMap;
use std::fs::read_to_string;

mod ngen_toml;
use ngen_toml::*;

mod parsed_config_processing;
use parsed_config_processing::config_table::*;
use parsed_config_processing::parsed_config_processing::*;
use parsed_config_processing::target::*;

/// ngen is a build file generator for the ninja build system.
#[derive(Parser)]
struct Args {
    /// Path to the toml configuration file
    #[arg(short, long, default_value = "ngen.toml")]
    config_file: String,

    /// Path to generated ninja file
    #[arg(short, long, default_value = "build.ninja")]
    output_file: String,

    #[arg(long, hide = true)]
    write_compile_commands: Option<String>,
}

fn main() {
    let args = Args::parse();

    let config_file = read_to_string(&args.config_file)
        .unwrap_or_else(|e| panic!("error reading config file `{}`: {e}", args.config_file));
    let toml_config: NgenToml = toml::from_str(&config_file)
        .unwrap_or_else(|e| panic!("error parsing config file `{}`: {e}", args.config_file));

    let config_table: ConfigTable;
    match &toml_config.config {
        Some(v) => {
            config_table = ConfigTable::from_parsed_config(&v).unwrap_or_else(|e| panic!("{e}"))
        }
        None => config_table = ConfigTable::get_default(),
    }

    let all_parsed_targets: &HashMap<String, ParsedTarget>;
    match &toml_config.targets {
        Some(v) => {
            all_parsed_targets = v;
        }
        None => panic!(
            "{}",
            ConfigurationError {
                message: "no targets found, nothing to do. Exiting".to_string()
            }
        ),
    }

    let mut targets: HashMap<&str, Target> = HashMap::new();
    for (name, parsed_target) in all_parsed_targets {
        if let Some(_) = targets.get(name as &str) {
            continue;
        }
        let mut inheritance_chain: HashMap<&str, &str> = HashMap::new();
        match Target::from_parsed_target(
            parsed_target,
            name,
            all_parsed_targets,
            &targets,
            &mut inheritance_chain,
        ) {
            Ok(v) => {
                targets.extend(v.into_iter());
            }
            Err(e) => panic!("{e}"),
        }
    }

    println!("{:#?}", targets);
}
